@@ -1,5 +1,5 @@
 /* Round towards positive infinity.
-   Copyright (C) 2007, 2010-2016 Free Software Foundation, Inc.
+   Copyright (C) 2007, 2010-2011 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
@@ -54,12 +54,6 @@
 # define MINUS_ZERO L_(-0.0)
 #endif
 
-/* MSVC with option -fp:strict refuses to compile constant initializers that
-   contain floating-point operations.  Pacify this compiler.  */
-#ifdef _MSC_VER
-# pragma fenv_access (off)
-#endif
-
 /* 2^(MANT_DIG-1).  */
 static const DOUBLE TWO_MANT_DIG =
   /* Assume MANT_DIG <= 5 * 31.
