@@ -1,6 +1,5 @@
-/* Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2004, 2006,
- *   2009, 2010, 2011, 2012, 2013, 2014, 2016 Free Software Foundation, Inc.
- *
+/* Copyright (C) 1996,1997,1998,1999,2000,2001, 2002, 2004, 2006, 2009, 2010, 2011 Free Software Foundation, Inc.
+ * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * as published by the Free Software Foundation; either version 3 of
@@ -35,7 +34,6 @@
 #endif
 
 #include <alloca.h>
-#include <dirname.h>
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -46,13 +44,13 @@
 #include "libguile/feature.h"
 #include "libguile/fports.h"
 #include "libguile/private-gc.h"  /* for SCM_MAX */
+#include "libguile/iselect.h"
 #include "libguile/strings.h"
 #include "libguile/vectors.h"
 #include "libguile/dynwind.h"
 
 #include "libguile/validate.h"
 #include "libguile/filesys.h"
-#include "libguile/load.h"	/* for scm_i_mirror_backslashes */
 
 
 #ifdef HAVE_IO_H
@@ -74,13 +72,17 @@
 # endif
 #endif
 
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
 #ifdef LIBC_H_WITH_UNISTD_H
 #include <libc.h>
 #endif
 
+#ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
+#endif
 
 #ifdef HAVE_STRING_H
 #include <string.h>
@@ -94,21 +96,40 @@
 #include <pwd.h>
 #endif
 
-#include <dirent.h>
 
-#define NAMLEN(dirent)  strlen ((dirent)->d_name)
-
-#ifdef HAVE_SYS_SENDFILE_H
-# include <sys/sendfile.h>
+#if HAVE_DIRENT_H
+# include <dirent.h>
+# define NAMLEN(dirent) strlen((dirent)->d_name)
+#else
+# define dirent direct
+# define NAMLEN(dirent) (dirent)->d_namlen
+# if HAVE_SYS_NDIR_H
+#  include <sys/ndir.h>
+# endif
+# if HAVE_SYS_DIR_H
+#  include <sys/dir.h>
+# endif
+# if HAVE_NDIR_H
+#  include <ndir.h>
+# endif
 #endif
 
-/* Glibc's `sendfile' function.  */
-#define sendfile_or_sendfile64			\
-  CHOOSE_LARGEFILE (sendfile, sendfile64)
-
-#include <full-read.h>
-#include <full-write.h>
+/* Some more definitions for the native Windows port. */
+#ifdef __MINGW32__
+# define mkdir(path, mode) mkdir (path)
+# define fsync(fd) _commit (fd)
+# define fchmod(fd, mode) (-1)
+#endif /* __MINGW32__ */
 
+/* dirfd() returns the file descriptor underlying a "DIR*" directory stream.
+   Found on MacOS X for instance.  The following definition is for Solaris
+   10, it's probably not right elsewhere, but that's ok, it shouldn't be
+   used elsewhere.  Crib note: If we need more then gnulib has a dirfd.m4
+   figuring out how to get the fd (dirfd function, dirfd macro, dd_fd field,
+   or d_fd field).  */
+#ifndef dirfd
+#define dirfd(dirstream) ((dirstream)->dd_fd)
+#endif
 
 
 
@@ -438,22 +459,35 @@ scm_stat2scm (struct stat_or_stat64 *stat_temp)
   return ans;
 }
 
-static int
-is_file_name_separator (SCM c)
-{
-  if (scm_is_eq (c, SCM_MAKE_CHAR ('/')))
-    return 1;
 #ifdef __MINGW32__
-  if (scm_is_eq (c, SCM_MAKE_CHAR ('\\')))
-    return 1;
-#endif
-  return 0;
+/*
+ * Try getting the appropiate stat buffer for a given file descriptor
+ * under Windows. It differentiates between file, pipe and socket 
+ * descriptors.
+ */
+static int fstat_Win32 (int fdes, struct stat *buf)
+{
+  int error, optlen = sizeof (int);
+
+  memset (buf, 0, sizeof (struct stat));
+
+  /* Is this a socket ? */
+  if (getsockopt (fdes, SOL_SOCKET, SO_ERROR, (void *) &error, &optlen) >= 0)
+    {
+      buf->st_mode = _S_IREAD | _S_IWRITE | _S_IEXEC;
+      buf->st_nlink = 1;
+      buf->st_atime = buf->st_ctime = buf->st_mtime = time (NULL);
+      return 0;
+    }
+  /* Maybe a regular file or pipe ? */
+  return fstat (fdes, buf);
 }
+#endif /* __MINGW32__ */
 
 SCM_DEFINE (scm_stat, "stat", 1, 1, 0, 
             (SCM object, SCM exception_on_error),
 	    "Return an object containing various information about the file\n"
-	    "determined by @var{object}.  @var{object} can be a string containing\n"
+	    "determined by @var{obj}.  @var{obj} can be a string containing\n"
 	    "a file name or a port or integer file descriptor which is open\n"
 	    "on a file (in which case @code{fstat} is used as the underlying\n"
 	    "system call).\n"
@@ -522,11 +556,21 @@ SCM_DEFINE (scm_stat, "stat", 1, 1, 0,
 
   if (scm_is_integer (object))
     {
+#ifdef __MINGW32__
+      SCM_SYSCALL (rv = fstat_Win32 (scm_to_int (object), &stat_temp));
+#else
       SCM_SYSCALL (rv = fstat_or_fstat64 (scm_to_int (object), &stat_temp));
+#endif
     }
   else if (scm_is_string (object))
     {
       char *file = scm_to_locale_string (object);
+#ifdef __MINGW32__
+      char *p;
+      p = file + strlen (file) - 1;
+      while (p > file && (*p == '/' || *p == '\\'))
+	*p-- = '\0';
+#endif
       SCM_SYSCALL (rv = stat_or_stat64 (file, &stat_temp));
       free (file);
     }
@@ -535,7 +579,11 @@ SCM_DEFINE (scm_stat, "stat", 1, 1, 0,
       object = SCM_COERCE_OUTPORT (object);
       SCM_VALIDATE_OPFPORT (1, object);
       fdes = SCM_FPORT_FDES (object);
+#ifdef __MINGW32__
+      SCM_SYSCALL (rv = fstat_Win32 (fdes, &stat_temp));
+#else
       SCM_SYSCALL (rv = fstat_or_fstat64 (fdes, &stat_temp));
+#endif
     }
 
   if (rv == -1)
@@ -555,11 +603,12 @@ SCM_DEFINE (scm_stat, "stat", 1, 1, 0,
 }
 #undef FUNC_NAME
 
+#ifdef HAVE_LSTAT
 SCM_DEFINE (scm_lstat, "lstat", 1, 0, 0, 
             (SCM str),
 	    "Similar to @code{stat}, but does not follow symbolic links, i.e.,\n"
 	    "it will return information about a symbolic link itself, not the\n"
-	    "file it points to.  @var{str} must be a string.")
+	    "file it points to.  @var{path} must be a string.")
 #define FUNC_NAME s_scm_lstat
 {
   int rv;
@@ -577,6 +626,7 @@ SCM_DEFINE (scm_lstat, "lstat", 1, 0, 0,
   return scm_stat2scm (&stat_temp);
 }
 #undef FUNC_NAME
+#endif /* HAVE_LSTAT */
 
 
 #ifdef HAVE_POSIX
@@ -584,6 +634,7 @@ SCM_DEFINE (scm_lstat, "lstat", 1, 0, 0,
 /* {Modifying Directories}
  */
 
+#ifdef HAVE_LINK
 SCM_DEFINE (scm_link, "link", 2, 0, 0,
             (SCM oldpath, SCM newpath),
 	    "Creates a new name @var{newpath} in the file system for the\n"
@@ -602,6 +653,7 @@ SCM_DEFINE (scm_link, "link", 2, 0, 0,
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
+#endif /* HAVE_LINK */
 
 
 /* {Navigating Directories}
@@ -610,7 +662,7 @@ SCM_DEFINE (scm_link, "link", 2, 0, 0,
 
 SCM_DEFINE (scm_chdir, "chdir", 1, 0, 0, 
             (SCM str),
-	    "Change the current working directory to @var{str}.\n"
+	    "Change the current working directory to @var{path}.\n"
 	    "The return value is unspecified.")
 #define FUNC_NAME s_scm_chdir
 {
@@ -625,13 +677,15 @@ SCM_DEFINE (scm_chdir, "chdir", 1, 0, 0,
 
 
 
+#ifdef HAVE_SELECT
+
 /* check that element is a port or file descriptor.  if it's a port
    and its buffer is ready for use, add it to the ports_ready list.
    otherwise add its file descriptor to *set.  the type of list can be
    determined from pos: SCM_ARG1 for reads, SCM_ARG2 for writes,
    SCM_ARG3 for excepts.  */
 static int
-set_element (fd_set *set, SCM *ports_ready, SCM element, int pos)
+set_element (SELECT_TYPE *set, SCM *ports_ready, SCM element, int pos)
 {
   int fd;
 
@@ -677,7 +731,7 @@ set_element (fd_set *set, SCM *ports_ready, SCM element, int pos)
    determined from pos: SCM_ARG1 for reads, SCM_ARG2 for writes,
    SCM_ARG3 for excepts.  */
 static int
-fill_select_type (fd_set *set, SCM *ports_ready, SCM list_or_vec, int pos)
+fill_select_type (SELECT_TYPE *set, SCM *ports_ready, SCM list_or_vec, int pos)
 {
   int max_fd = 0;
 
@@ -712,7 +766,7 @@ fill_select_type (fd_set *set, SCM *ports_ready, SCM list_or_vec, int pos)
 /* if element (a file descriptor or port) appears in *set, cons it to
    list.  return list.  */
 static SCM
-get_element (fd_set *set, SCM element, SCM list)
+get_element (SELECT_TYPE *set, SCM element, SCM list)
 {
   int fd;
 
@@ -738,7 +792,7 @@ get_element (fd_set *set, SCM element, SCM list)
    *set and appending them to ports_ready.  result is converted to a
    vector if list_or_vec is a vector.  */
 static SCM 
-retrieve_select_type (fd_set *set, SCM ports_ready, SCM list_or_vec)
+retrieve_select_type (SELECT_TYPE *set, SCM ports_ready, SCM list_or_vec)
 {
   SCM answer_list = ports_ready;
 
@@ -772,13 +826,8 @@ SCM_DEFINE (scm_select, "select", 3, 2, 0,
 	    "This procedure has a variety of uses: waiting for the ability\n"
 	    "to provide input, accept output, or the existence of\n"
 	    "exceptional conditions on a collection of ports or file\n"
-	    "descriptors, or waiting for a timeout to occur.\n\n"
-
-	    "When an error occurs, of if it is interrupted by a signal, this\n"
-	    "procedure throws a @code{system-error} exception\n"
-	    "(@pxref{Conventions, @code{system-error}}).  In case of an\n"
-	    "interruption, the associated error number is @var{EINTR}.\n\n"
-
+	    "descriptors, or waiting for a timeout to occur.\n"
+	    "It also returns if interrupted by a signal.\n\n"
 	    "@var{reads}, @var{writes} and @var{excepts} can be lists or\n"
 	    "vectors, with each member a port or a file descriptor.\n"
 	    "The value returned is a list of three corresponding\n"
@@ -804,9 +853,9 @@ SCM_DEFINE (scm_select, "select", 3, 2, 0,
 {
   struct timeval timeout;
   struct timeval * time_ptr;
-  fd_set read_set;
-  fd_set write_set;
-  fd_set except_set;
+  SELECT_TYPE read_set;
+  SELECT_TYPE write_set;
+  SELECT_TYPE except_set;
   int read_count;
   int write_count;
   int except_count;
@@ -897,9 +946,9 @@ SCM_DEFINE (scm_select, "select", 3, 2, 0,
     }
 
   {
-    int rv = select (max_fd + 1,
-                     &read_set, &write_set, &except_set,
-                     time_ptr);
+    int rv = scm_std_select (max_fd + 1,
+			     &read_set, &write_set, &except_set,
+			     time_ptr);
     if (rv < 0)
       SCM_SYSERROR;
   }
@@ -908,16 +957,17 @@ SCM_DEFINE (scm_select, "select", 3, 2, 0,
 		     retrieve_select_type (&except_set, SCM_EOL, excepts));
 }
 #undef FUNC_NAME
+#endif /* HAVE_SELECT */
 
 
 
 #ifdef HAVE_FCNTL
 SCM_DEFINE (scm_fcntl, "fcntl", 2, 1, 0,
             (SCM object, SCM cmd, SCM value),
-	    "Apply @var{cmd} to the specified file descriptor or the underlying\n"
+	    "Apply @var{command} to the specified file descriptor or the underlying\n"
 	    "file descriptor of the specified port.  @var{value} is an optional\n"
 	    "integer argument.\n\n"
-	    "Values for @var{cmd} are:\n\n"
+	    "Values for @var{command} are:\n\n"
 	    "@table @code\n"
 	    "@item F_DUPFD\n"
 	    "Duplicate a file descriptor\n"
@@ -965,9 +1015,9 @@ SCM_DEFINE (scm_fcntl, "fcntl", 2, 1, 0,
 
 SCM_DEFINE (scm_fsync, "fsync", 1, 0, 0, 
             (SCM object),
-	    "Copies any unwritten data for the specified output file\n"
-	    "descriptor to disk.  If @var{object} is a port, its buffer is\n"
-	    "flushed before the underlying file descriptor is fsync'd.\n"
+	    "Copies any unwritten data for the specified output file descriptor to disk.\n"
+	    "If @var{port/fd} is a port, its buffer is flushed before the underlying\n"
+	    "file descriptor is fsync'd.\n"
 	    "The return value is unspecified.")
 #define FUNC_NAME s_scm_fsync
 {
@@ -992,9 +1042,8 @@ SCM_DEFINE (scm_fsync, "fsync", 1, 0, 0,
 #ifdef HAVE_SYMLINK
 SCM_DEFINE (scm_symlink, "symlink", 2, 0, 0,
             (SCM oldpath, SCM newpath),
-	    "Create a symbolic link named @var{newpath} with the value\n"
-	    "(i.e., pointing to) @var{oldpath}.  The return value is\n"
-	    "unspecified.")
+	    "Create a symbolic link named @var{path-to} with the value (i.e., pointing to)\n"
+	    "@var{path-from}.  The return value is unspecified.")
 #define FUNC_NAME s_scm_symlink
 {
   int val;
@@ -1009,6 +1058,7 @@ SCM_DEFINE (scm_symlink, "symlink", 2, 0, 0,
 #undef FUNC_NAME
 #endif /* HAVE_SYMLINK */
 
+#ifdef HAVE_READLINK
 SCM_DEFINE (scm_readlink, "readlink", 1, 0, 0, 
             (SCM path),
 	    "Return the value of the symbolic link named by @var{path} (a\n"
@@ -1047,10 +1097,11 @@ SCM_DEFINE (scm_readlink, "readlink", 1, 0, 0,
   return result;
 }
 #undef FUNC_NAME
+#endif /* HAVE_READLINK */
 
 SCM_DEFINE (scm_copy_file, "copy-file", 2, 0, 0,
             (SCM oldfile, SCM newfile),
-	    "Copy the file specified by @var{oldfile} to @var{newfile}.\n"
+	    "Copy the file specified by @var{path-from} to @var{path-to}.\n"
 	    "The return value is unspecified.")
 #define FUNC_NAME s_scm_copy_file
 {
@@ -1067,11 +1118,15 @@ SCM_DEFINE (scm_copy_file, "copy-file", 2, 0, 0,
   c_newfile = scm_to_locale_string (newfile);
   scm_dynwind_free (c_newfile);
 
-  oldfd = open_or_open64 (c_oldfile, O_RDONLY | O_BINARY);
+  oldfd = open_or_open64 (c_oldfile, O_RDONLY);
   if (oldfd == -1)
     SCM_SYSERROR;
 
+#ifdef __MINGW32__
+  SCM_SYSCALL (rv = fstat_Win32 (oldfd, &oldstat));
+#else
   SCM_SYSCALL (rv = fstat_or_fstat64 (oldfd, &oldstat));
+#endif
   if (rv == -1)
     goto err_close_oldfd;
 
@@ -1101,117 +1156,6 @@ SCM_DEFINE (scm_copy_file, "copy-file", 2, 0, 0,
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (scm_sendfile, "sendfile", 3, 1, 0,
-	    (SCM out, SCM in, SCM count, SCM offset),
-	    "Send @var{count} bytes from @var{in} to @var{out}, both of which "
-	    "must be either open file ports or file descriptors.  When "
-	    "@var{offset} is omitted, start reading from @var{in}'s current "
-	    "position; otherwise, start reading at @var{offset}.  Return "
-	    "the number of bytes actually sent.")
-#define FUNC_NAME s_scm_sendfile
-{
-#define VALIDATE_FD_OR_PORT(cvar, svar, pos)	\
-  if (scm_is_integer (svar))			\
-    cvar = scm_to_int (svar);			\
-  else						\
-    {						\
-      SCM_VALIDATE_OPFPORT (pos, svar);		\
-      scm_flush (svar);				\
-      cvar = SCM_FPORT_FDES (svar);		\
-    }
-
-  ssize_t result SCM_UNUSED;
-  size_t c_count, total = 0;
-  scm_t_off c_offset;
-  int in_fd, out_fd;
-
-  VALIDATE_FD_OR_PORT (out_fd, out, 1);
-  VALIDATE_FD_OR_PORT (in_fd, in, 2);
-  c_count = scm_to_size_t (count);
-  c_offset = SCM_UNBNDP (offset) ? 0 : scm_to_off_t (offset);
-
-#if defined HAVE_SYS_SENDFILE_H && defined HAVE_SENDFILE
-  /* The Linux-style sendfile(2), which is different from the BSD-style.  */
-
-  {
-    off_t *offset_ptr;
-
-    offset_ptr = SCM_UNBNDP (offset) ? NULL : &c_offset;
-
-    /* On Linux, when OUT_FD is a file, everything is transferred at once and
-       RESULT == C_COUNT.  However, when OUT_FD is a pipe or other "slow"
-       device, fewer bytes may be transferred, hence the loop.  RESULT == 0
-       means EOF on IN_FD, so leave the loop in that case.  */
-    do
-      {
-	result = sendfile_or_sendfile64 (out_fd, in_fd, offset_ptr,
-					 c_count - total);
-	if (result > 0)
-	  /* At this point, either OFFSET_PTR is non-NULL and it has been
-	     updated to the current offset in IN_FD, or it is NULL and IN_FD's
-	     offset has been updated.  */
-	  total += result;
-	else if (result < 0 && (errno == EINTR || errno == EAGAIN))
-	  /* Keep going.  */
-	  result = 1;
-      }
-    while (total < c_count && result > 0);
-  }
-
-  /* Quoting the Linux man page: "In Linux kernels before 2.6.33, out_fd
-     must refer to a socket.  Since Linux 2.6.33 it can be any file."
-     Fall back to read(2) and write(2) when such an error occurs.  */
-  if (result < 0 && errno != EINVAL && errno != ENOSYS)
-    SCM_SYSERROR;
-  else if (result < 0)
-#endif
-  {
-    char buf[8192];
-    size_t left;
-    int reached_eof = 0;
-
-    if (!SCM_UNBNDP (offset))
-      {
-	if (SCM_PORTP (in))
-	  scm_seek (in, scm_from_off_t (c_offset), scm_from_int (SEEK_SET));
-	else
-	  {
-	    if (lseek_or_lseek64 (in_fd, c_offset, SEEK_SET) < 0)
-	      SCM_SYSERROR;
-	  }
-      }
-
-    for (total = 0, left = c_count; total < c_count && !reached_eof; )
-      {
-	size_t asked, obtained, written;
-
-	asked = SCM_MIN (sizeof buf, left);
-	obtained = full_read (in_fd, buf, asked);
-	if (obtained < asked)
-          {
-            if (errno == 0)
-              reached_eof = 1;
-            else
-              SCM_SYSERROR;
-          }
-
-	left -= obtained;
-
-	written = full_write (out_fd, buf, obtained);
-	if (written < obtained)
-	  SCM_SYSERROR;
-
-	total += written;
-      }
-
-  }
-
-  return scm_from_size_t (total);
-
-#undef VALIDATE_FD_OR_PORT
-}
-#undef FUNC_NAME
-
 #endif /* HAVE_POSIX */
 
 
@@ -1242,9 +1186,6 @@ SCM_DEFINE (scm_getcwd, "getcwd", 0, 0, 0,
       errno = save_errno;
       SCM_SYSERROR;
     }
-  /* On Windows, convert backslashes in current directory to forward
-     slashes.  */
-  scm_i_mirror_backslashes (wd);
   result = scm_from_locale_stringn (wd, strlen (wd));
   free (wd);
   return result;
@@ -1252,28 +1193,36 @@ SCM_DEFINE (scm_getcwd, "getcwd", 0, 0, 0,
 #undef FUNC_NAME
 #endif /* HAVE_GETCWD */
 
+#ifdef HAVE_MKDIR
 SCM_DEFINE (scm_mkdir, "mkdir", 1, 1, 0,
             (SCM path, SCM mode),
 	    "Create a new directory named by @var{path}.  If @var{mode} is omitted\n"
-	    "then the permissions of the directory are set to @code{#o777}\n"
-	    "masked with the current umask (@pxref{Processes, @code{umask}}).\n"
-	    "Otherwise they are set to the value specified with @var{mode}.\n"
-	    "The return value is unspecified.")
+	    "then the permissions of the directory file are set using the current\n"
+	    "umask.  Otherwise they are set to the decimal value specified with\n"
+	    "@var{mode}.  The return value is unspecified.")
 #define FUNC_NAME s_scm_mkdir
 {
   int rv;
-  mode_t c_mode;
+  mode_t mask;
 
-  c_mode = SCM_UNBNDP (mode) ? 0777 : scm_to_uint (mode);
-
-  STRING_SYSCALL (path, c_path, rv = mkdir (c_path, c_mode));
+  if (SCM_UNBNDP (mode))
+    {
+      mask = umask (0);
+      umask (mask);
+      STRING_SYSCALL (path, c_path, rv = mkdir (c_path, 0777 ^ mask));
+    }
+  else
+    {
+      STRING_SYSCALL (path, c_path, rv = mkdir (c_path, scm_to_uint (mode)));
+    }
   if (rv != 0)
     SCM_SYSERROR;
-
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
+#endif /* HAVE_MKDIR */
 
+#ifdef HAVE_RMDIR
 SCM_DEFINE (scm_rmdir, "rmdir", 1, 0, 0, 
             (SCM path),
 	    "Remove the existing directory named by @var{path}.  The directory must\n"
@@ -1288,6 +1237,27 @@ SCM_DEFINE (scm_rmdir, "rmdir", 1, 0, 0,
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
+#endif
+
+#ifdef HAVE_RENAME
+#define my_rename rename
+#else
+static int
+my_rename (const char *oldname, const char *newname)
+{
+  int rv;
+
+  SCM_SYSCALL (rv = link (oldname, newname));
+  if (rv == 0)
+    {
+      SCM_SYSCALL (rv = unlink (oldname));
+      if (rv != 0)
+	/* unlink failed.  remove new name */
+	SCM_SYSCALL (unlink (newname)); 
+    }
+  return rv;
+}
+#endif
 
 SCM_DEFINE (scm_rename, "rename-file", 2, 0, 0,
             (SCM oldname, SCM newname),
@@ -1299,7 +1269,7 @@ SCM_DEFINE (scm_rename, "rename-file", 2, 0, 0,
 
   STRING2_SYSCALL (oldname, c_oldname,
 		   newname, c_newname,
-		   rv = rename (c_oldname, c_newname));
+		   rv = my_rename (c_oldname, c_newname));
   if (rv != 0)
     SCM_SYSERROR;
   return SCM_UNSPECIFIED;
@@ -1309,7 +1279,7 @@ SCM_DEFINE (scm_rename, "rename-file", 2, 0, 0,
 
 SCM_DEFINE (scm_delete_file, "delete-file", 1, 0, 0, 
            (SCM str),
-	    "Deletes (or \"unlinks\") the file specified by @var{str}.")
+	    "Deletes (or \"unlinks\") the file specified by @var{path}.")
 #define FUNC_NAME s_scm_delete_file
 {
   int ans;
@@ -1377,23 +1347,22 @@ SCM_DEFINE (scm_access, "access?", 2, 0, 0,
 
 SCM_DEFINE (scm_chmod, "chmod", 2, 0, 0,
             (SCM object, SCM mode),
-	    "Changes the permissions of the file referred to by\n"
-	    "@var{object}.  @var{object} can be a string containing a file\n"
-	    "name or a port or integer file descriptor which is open on a\n"
-	    "file (in which case @code{fchmod} is used as the underlying\n"
-	    "system call).  @var{mode} specifies the new permissions as a\n"
-	    "decimal number, e.g., @code{(chmod \"foo\" #o755)}.\n"
+	    "Changes the permissions of the file referred to by @var{obj}.\n"
+	    "@var{obj} can be a string containing a file name or a port or integer file\n"
+	    "descriptor which is open on a file (in which case @code{fchmod} is used\n"
+	    "as the underlying system call).\n"
+	    "@var{mode} specifies\n"
+	    "the new permissions as a decimal number, e.g., @code{(chmod \"foo\" #o755)}.\n"
 	    "The return value is unspecified.")
 #define FUNC_NAME s_scm_chmod
 {
   int rv;
+  int fdes;
 
   object = SCM_COERCE_OUTPORT (object);
 
-#if HAVE_FCHMOD
   if (scm_is_integer (object) || SCM_OPFPORTP (object))
     {
-      int fdes;
       if (scm_is_integer (object))
 	fdes = scm_to_int (object);
       else
@@ -1401,7 +1370,6 @@ SCM_DEFINE (scm_chmod, "chmod", 2, 0, 0,
       SCM_SYSCALL (rv = fchmod (fdes, scm_to_int (mode)));
     }
   else
-#endif
     {
       STRING_SYSCALL (object, c_object,
 		      rv = chmod (c_object, scm_to_int (mode)));
@@ -1434,9 +1402,12 @@ SCM_DEFINE (scm_umask, "umask", 0, 1, 0,
 }
 #undef FUNC_NAME
 
-SCM_INTERNAL SCM scm_i_mkstemp (SCM, SCM);
-SCM_DEFINE (scm_i_mkstemp, "mkstemp!", 1, 1, 0,
-	    (SCM tmpl, SCM mode),
+#ifndef HAVE_MKSTEMP
+extern int mkstemp (char *);
+#endif
+
+SCM_DEFINE (scm_mkstemp, "mkstemp!", 1, 0, 0,
+	    (SCM tmpl),
 	    "Create a new unique file in the file system and return a new\n"
 	    "buffered port open for reading and writing to the file.\n"
 	    "\n"
@@ -1455,38 +1426,18 @@ SCM_DEFINE (scm_i_mkstemp, "mkstemp!", 1, 1, 0,
 	    "(let ((port (mkstemp! (string-copy \"/tmp/myfile-XXXXXX\"))))\n"
 	    "  (chmod port (logand #o666 (lognot (umask))))\n"
 	    "  ...)\n"
-	    "@end example\n"
-            "\n"
-            "The optional @var{mode} argument specifies a mode, as a string\n"
-            "in the same format that @code{open-file} takes.  It defaults\n"
-            "to @code{\"w+\"}.")
-#define FUNC_NAME s_scm_i_mkstemp
+	    "@end example")
+#define FUNC_NAME s_scm_mkstemp
 {
   char *c_tmpl;
-  long mode_bits;
   int rv;
-  int open_flags, is_binary;
-  SCM port;
 
   scm_dynwind_begin (0);
 
   c_tmpl = scm_to_locale_string (tmpl);
   scm_dynwind_free (c_tmpl);
-  if (SCM_UNBNDP (mode))
-    {
-      /* mkostemp will create a read/write file and add on additional
-         flags; open_flags just adjoins flags to that set.  */
-      open_flags = 0;
-      is_binary = 0;
-      mode_bits = SCM_OPN | SCM_RDNG | SCM_WRTNG;
-    }
-  else
-    {
-      open_flags = scm_i_mode_to_open_flags (mode, &is_binary, FUNC_NAME);
-      mode_bits = scm_i_mode_bits (mode);
-    }
 
-  SCM_SYSCALL (rv = mkostemp (c_tmpl, open_flags));
+  SCM_SYSCALL (rv = mkstemp (c_tmpl));
   if (rv == -1)
     SCM_SYSERROR;
 
@@ -1495,45 +1446,15 @@ SCM_DEFINE (scm_i_mkstemp, "mkstemp!", 1, 1, 0,
 			tmpl, SCM_INUM0);
 
   scm_dynwind_end ();
-
-  port = scm_i_fdes_to_port (rv, mode_bits, tmpl);
-  if (is_binary)
-    /* Use the binary-friendly ISO-8859-1 encoding. */
-    scm_i_set_port_encoding_x (port, NULL);
-
-  return port;
+  return scm_fdes_to_port (rv, "w+", tmpl);
 }
 #undef FUNC_NAME
 
-SCM
-scm_mkstemp (SCM tmpl)
-{
-  return scm_i_mkstemp (tmpl, SCM_UNDEFINED);
-}
-
 
 /* Filename manipulation */
 
 SCM scm_dot_string;
 
-#ifdef __MINGW32__
-SCM_SYMBOL (sym_file_name_convention, "windows");
-#else
-SCM_SYMBOL (sym_file_name_convention, "posix");
-#endif
-
-SCM_INTERNAL SCM scm_system_file_name_convention (void);
-
-SCM_DEFINE (scm_system_file_name_convention,
-            "system-file-name-convention", 0, 0, 0, (void),
-	    "Return either @code{posix} or @code{windows}, depending on\n"
-            "what kind of system this Guile is running on.")
-#define FUNC_NAME s_scm_system_file_name_convention
-{
-  return sym_file_name_convention;
-}
-#undef FUNC_NAME
-
 SCM_DEFINE (scm_dirname, "dirname", 1, 0, 0, 
             (SCM filename),
 	    "Return the directory name component of the file name\n"
@@ -1541,22 +1462,46 @@ SCM_DEFINE (scm_dirname, "dirname", 1, 0, 0,
 	    "component, @code{.} is returned.")
 #define FUNC_NAME s_scm_dirname
 {
-  char *c_filename, *c_dirname;
-  SCM res;
+  long int i;
+  unsigned long int len;
 
-  scm_dynwind_begin (0);
-  c_filename = scm_to_utf8_string (filename);
-  scm_dynwind_free (c_filename);
+  SCM_VALIDATE_STRING (1, filename);
 
-  c_dirname = mdir_name (c_filename);
-  if (!c_dirname)
-    SCM_SYSERROR;
-  scm_dynwind_free (c_dirname);
+  len = scm_i_string_length (filename);
 
-  res = scm_from_utf8_string (c_dirname);
-  scm_dynwind_end ();
-
-  return res;
+  i = len - 1;
+#ifdef __MINGW32__
+  while (i >= 0 && (scm_i_string_ref (filename, i) == '/'
+		    || scm_i_string_ref (filename, i) == '\\')) 
+    --i;
+  while (i >= 0 && (scm_i_string_ref (filename, i) != '/'
+		    && scm_i_string_ref (filename, i) != '\\')) 
+    --i;
+  while (i >= 0 && (scm_i_string_ref (filename, i) == '/'
+		    || scm_i_string_ref (filename, i) == '\\')) 
+    --i;
+#else
+  while (i >= 0 && scm_i_string_ref (filename, i) == '/') 
+    --i;
+  while (i >= 0 && scm_i_string_ref (filename, i) != '/') 
+    --i;
+  while (i >= 0 && scm_i_string_ref (filename, i) == '/') 
+    --i;
+#endif /* ndef __MINGW32__ */
+  if (i < 0)
+    {
+#ifdef __MINGW32__
+      if (len > 0 && (scm_i_string_ref (filename, 0) == '/'
+		      || scm_i_string_ref (filename, 0) == '\\'))
+#else
+      if (len > 0 && scm_i_string_ref (filename, 0) == '/')
+#endif /* ndef __MINGW32__ */
+	return scm_c_substring (filename, 0, 1);
+      else
+	return scm_dot_string;
+    }
+  else
+    return scm_c_substring (filename, 0, i + 1);
 }
 #undef FUNC_NAME
 
@@ -1565,31 +1510,62 @@ SCM_DEFINE (scm_basename, "basename", 1, 1, 0,
 	    "Return the base name of the file name @var{filename}. The\n"
 	    "base name is the file name without any directory components.\n"
 	    "If @var{suffix} is provided, and is equal to the end of\n"
-	    "@var{filename}, it is removed also.")
+	    "@var{basename}, it is removed also.")
 #define FUNC_NAME s_scm_basename
 {
-  char *c_filename, *c_last_component;
-  SCM res;
+  int i, j, len, end;
 
-  scm_dynwind_begin (0);
-  c_filename = scm_to_utf8_string (filename);
-  scm_dynwind_free (c_filename);
+  SCM_VALIDATE_STRING (1, filename);
+  len = scm_i_string_length (filename);
 
-  c_last_component = last_component (c_filename);
-  if (!c_last_component)
-    res = filename;
+  if (SCM_UNBNDP (suffix))
+    j = -1;
   else
-    res = scm_from_utf8_string (c_last_component);
-  scm_dynwind_end ();
-
-  if (!SCM_UNBNDP (suffix) &&
-      scm_is_true (scm_string_suffix_p (suffix, filename,
-                                        SCM_UNDEFINED, SCM_UNDEFINED,
-                                        SCM_UNDEFINED, SCM_UNDEFINED)))
-    res = scm_c_substring
-      (res, 0, scm_c_string_length (res) - scm_c_string_length (suffix));
-
-  return res;
+    {
+      SCM_VALIDATE_STRING (2, suffix);
+      j = scm_i_string_length (suffix) - 1;
+    }
+  i = len - 1;
+#ifdef __MINGW32__
+  while (i >= 0 && (scm_i_string_ref (filename, i) == '/'
+		    || scm_i_string_ref (filename, i) ==  '\\'))
+    --i;
+#else
+  while (i >= 0 && scm_i_string_ref (filename, i) == '/')
+    --i;
+#endif /* ndef __MINGW32__ */
+  end = i;
+  while (i >= 0 && j >= 0 
+	 && (scm_i_string_ref (filename, i)
+	     == scm_i_string_ref (suffix, j)))
+    {
+      --i;
+      --j;
+    }
+  if (j == -1)
+    end = i;
+#ifdef __MINGW32__
+  while (i >= 0 && (scm_i_string_ref (filename, i) != '/'
+		    && scm_i_string_ref (filename, i) != '\\'))
+    --i;
+#else
+  while (i >= 0 && scm_i_string_ref (filename, i) != '/')
+    --i;
+#endif /* ndef __MINGW32__ */
+  if (i == end)
+    {
+#ifdef __MINGW32__
+      if (len > 0 && (scm_i_string_ref (filename, 0) ==  '/'
+		      || scm_i_string_ref (filename, 0) ==  '\\'))
+#else
+      if (len > 0 && scm_i_string_ref (filename, 0) == '/')
+#endif /* ndef __MINGW32__ */
+        return scm_c_substring (filename, 0, 1);
+      else
+	return scm_dot_string;
+    }
+  else
+    return scm_c_substring (filename, i+1, end+1);
 }
 #undef FUNC_NAME
 
@@ -1632,33 +1608,32 @@ scm_i_relativize_path (SCM path, SCM in_path)
   scanon = scm_take_locale_string (canon);
 
   for (; scm_is_pair (in_path); in_path = scm_cdr (in_path))
-    {
-      SCM dir = scm_car (in_path);
-      size_t len = scm_c_string_length (dir);
+    if (scm_is_true (scm_string_prefix_p (scm_car (in_path),
+                                          scanon,
+                                          SCM_UNDEFINED, SCM_UNDEFINED,
+                                          SCM_UNDEFINED, SCM_UNDEFINED)))
+      {
+        size_t len = scm_c_string_length (scm_car (in_path));
 
-      /* When DIR is empty, it means "current working directory".  We
-	 could set DIR to (getcwd) in that case, but then the
-	 canonicalization would depend on the current directory, which
-	 is not what we want in the context of `compile-file', for
-	 instance.  */
-      if (len > 0
-	  && scm_is_true (scm_string_prefix_p (dir, scanon,
-					       SCM_UNDEFINED, SCM_UNDEFINED,
-					       SCM_UNDEFINED, SCM_UNDEFINED)))
-	{
-	  /* DIR either has a trailing delimiter or doesn't.  SCANON
-	     will be delimited by single delimiters.  When DIR does not
-	     have a trailing delimiter, add one to the length to strip
-	     off the delimiter within SCANON.  */
-	  if (!is_file_name_separator (scm_c_string_ref (dir, len - 1)))
-	    len++;
+        /* The path either has a trailing delimiter or doesn't. scanon will be
+           delimited by single delimiters. In the case in which the path does
+           not have a trailing delimiter, add one to the length to strip off the
+           delimiter within scanon. */
+        if (!len
+#ifdef __MINGW32__
+            || (scm_i_string_ref (scm_car (in_path), len - 1) != '/'
+                && scm_i_string_ref (scm_car (in_path), len - 1) != '\\')
+#else
+            || scm_i_string_ref (scm_car (in_path), len - 1) != '/'
+#endif
+            )
+          len++;
 
-	  if (scm_c_string_length (scanon) > len)
-	    return scm_substring (scanon, scm_from_size_t (len), SCM_UNDEFINED);
-	  else
-	    return SCM_BOOL_F;
-	}
-    }
+        if (scm_c_string_length (scanon) > len)
+          return scm_substring (scanon, scm_from_size_t (len), SCM_UNDEFINED);
+        else
+          return SCM_BOOL_F;
+      }
 
   return SCM_BOOL_F;
 }
@@ -1672,7 +1647,7 @@ scm_t_bits scm_tc16_dir;
 
 SCM_DEFINE (scm_directory_stream_p, "directory-stream?", 1, 0, 0,
 	    (SCM obj),
-	    "Return a boolean indicating whether @var{obj} is a directory\n"
+	    "Return a boolean indicating whether @var{object} is a directory\n"
 	    "stream as returned by @code{opendir}.")
 #define FUNC_NAME s_scm_directory_stream_p
 {
@@ -1683,7 +1658,7 @@ SCM_DEFINE (scm_directory_stream_p, "directory-stream?", 1, 0, 0,
 
 SCM_DEFINE (scm_opendir, "opendir", 1, 0, 0,
 	    (SCM dirname),
-	    "Open the directory specified by @var{dirname} and return a directory\n"
+	    "Open the directory specified by @var{path} and return a directory\n"
 	    "stream.")
 #define FUNC_NAME s_scm_opendir
 {
@@ -1705,7 +1680,7 @@ SCM_DEFINE (scm_opendir, "opendir", 1, 0, 0,
 SCM_DEFINE (scm_readdir, "readdir", 1, 0, 0,
 	    (SCM port),
 	    "Return (as a string) the next directory entry from the directory stream\n"
-	    "@var{port}.  If there is no remaining entry to be read then the\n"
+	    "@var{stream}.  If there is no remaining entry to be read then the\n"
 	    "end of file object is returned.")
 #define FUNC_NAME s_scm_readdir
 {
@@ -1779,7 +1754,7 @@ SCM_DEFINE (scm_readdir, "readdir", 1, 0, 0,
 
 SCM_DEFINE (scm_rewinddir, "rewinddir", 1, 0, 0,
 	    (SCM port),
-	    "Reset the directory port @var{port} so that the next call to\n"
+	    "Reset the directory port @var{stream} so that the next call to\n"
 	    "@code{readdir} will return the first directory entry.")
 #define FUNC_NAME s_scm_rewinddir
 {
@@ -1796,7 +1771,7 @@ SCM_DEFINE (scm_rewinddir, "rewinddir", 1, 0, 0,
 
 SCM_DEFINE (scm_closedir, "closedir", 1, 0, 0,
 	    (SCM port),
-	    "Close the directory stream @var{port}.\n"
+	    "Close the directory stream @var{stream}.\n"
 	    "The return value is unspecified.")
 #define FUNC_NAME s_scm_closedir
 {
