@@ -1,4 +1,4 @@
-/* Copyright (C) 1998,1999,2000,2001,2002,2003,2004,2008,2009,2010,2011,2012,2014,2015
+/* Copyright (C) 1998,1999,2000,2001,2002,2003,2004,2008,2009,2010,2011
  * Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
@@ -168,8 +168,6 @@ static SCM class_vm;
 static SCM class_vm_cont;
 static SCM class_bytevector;
 static SCM class_uvec;
-static SCM class_array;
-static SCM class_bitvector;
 
 static SCM vtable_class_map = SCM_BOOL_F;
 static scm_i_pthread_mutex_t vtable_class_map_lock = SCM_I_PTHREAD_MUTEX_INITIALIZER;
@@ -290,10 +288,6 @@ SCM_DEFINE (scm_class_of, "class-of", 1, 0, 0,
             return class_bytevector;
           else
             return class_uvec;
-	case scm_tc7_array:
-          return class_array;
-	case scm_tc7_bitvector:
-          return class_bitvector;
 	case scm_tc7_string:
 	  return scm_class_string;
         case scm_tc7_number:
@@ -659,7 +653,7 @@ SCM_DEFINE (scm_sys_initialize_object, "%initialize-object", 2, 0, 0,
        get_n_set = SCM_CDR (get_n_set), slots = SCM_CDR (slots))
     {
       SCM slot_name  = SCM_CAR (slots);
-      SCM slot_value = SCM_GOOPS_UNBOUND;
+      SCM slot_value = SCM_PACK (0);
 
       if (!scm_is_null (SCM_CDR (slot_name)))
 	{
@@ -683,12 +677,12 @@ SCM_DEFINE (scm_sys_initialize_object, "%initialize-object", 2, 0, 0,
 	      slot_value = scm_i_get_keyword (tmp,
 					      initargs,
 					      n_initargs,
-					      SCM_GOOPS_UNBOUND,
+					      SCM_PACK (0),
 					      FUNC_NAME);
 	    }
 	}
 
-      if (!SCM_GOOPS_UNBOUNDP (slot_value))
+      if (SCM_UNPACK (slot_value))
 	/* set slot to provided value */
 	set_slot_value (class, obj, SCM_CAR (get_n_set), slot_value);
       else
@@ -696,10 +690,14 @@ SCM_DEFINE (scm_sys_initialize_object, "%initialize-object", 2, 0, 0,
 	  /* set slot to its :init-form if it exists */
 	  tmp = SCM_CADAR (get_n_set);
 	  if (scm_is_true (tmp))
-            set_slot_value (class,
-                            obj,
-                            SCM_CAR (get_n_set),
-                            scm_call_0 (tmp));
+	    {
+	      slot_value = get_slot_value (class, obj, SCM_CAR (get_n_set));
+	      if (SCM_GOOPS_UNBOUNDP (slot_value))
+                set_slot_value (class,
+                                obj,
+                                SCM_CAR (get_n_set),
+                                scm_call_0 (tmp));
+	    }
 	}
     }
 
@@ -978,7 +976,7 @@ create_basic_classes (void)
   /**** <class> ****/
   SCM cs = scm_from_locale_string (SCM_CLASS_CLASS_LAYOUT);
   SCM name = scm_from_latin1_symbol ("<class>");
-  scm_class_class = scm_i_make_vtable_vtable (cs);
+  scm_class_class = scm_make_vtable_vtable (cs, SCM_INUM0, SCM_EOL);
   SCM_SET_CLASS_FLAGS (scm_class_class, (SCM_CLASSF_GOOPS_OR_VALID
 					 | SCM_CLASSF_METACLASS));
 
@@ -1759,22 +1757,15 @@ scm_call_generic_3 (SCM gf, SCM a1, SCM a2, SCM a3)
   return scm_call_3 (SCM_STRUCT_PROCEDURE (gf), a1, a2, a3);
 }
 
-static SCM delayed_compile_var;
-
-static void
-init_delayed_compile_var (void)
-{
-  delayed_compile_var
-    = scm_c_private_lookup ("oop goops dispatch", "delayed-compile");
-}
-
+SCM_SYMBOL (sym_delayed_compile, "delayed-compile");
 static SCM
 make_dispatch_procedure (SCM gf)
 {
-  static scm_i_pthread_once_t once = SCM_I_PTHREAD_ONCE_INIT;
-  scm_i_pthread_once (&once, init_delayed_compile_var);
-
-  return scm_call_1 (scm_variable_ref (delayed_compile_var), gf);
+  static SCM var = SCM_BOOL_F;
+  if (scm_is_false (var))
+    var = scm_module_variable (scm_c_resolve_module ("oop goops dispatch"),
+                               sym_delayed_compile);
+  return scm_call_1 (SCM_VARIABLE_REF (var), gf);
 }
 
 static void
@@ -2053,11 +2044,6 @@ sort_applicable_methods (SCM method_list, long size, SCM const *targs)
   return scm_vector_to_list (vector);
 }
 
-static int
-is_accessor_method (SCM method) {
-  return SCM_IS_A_P (method, scm_class_accessor_method);
-}
-
 SCM
 scm_compute_applicable_methods (SCM gf, SCM args, long len, int find_method_p)
 {
@@ -2093,10 +2079,6 @@ scm_compute_applicable_methods (SCM gf, SCM args, long len, int find_method_p)
   for (l = scm_generic_function_methods (gf); !scm_is_null (l); l = SCM_CDR (l))
     {
       fl = SPEC_OF (SCM_CAR (l));
-      /* Only accept accessors which match exactly in first arg. */
-      if ((scm_is_null (fl) || !scm_is_eq (types[0], SCM_CAR (fl)))
-          && is_accessor_method (SCM_CAR (l)))
-	continue;
       for (i = 0; ; i++, fl = SCM_CDR (fl))
 	{
 	  if (SCM_INSTANCEP (fl)
@@ -2541,10 +2523,6 @@ create_standard_classes (void)
 	       scm_class_class, scm_class_top,		   SCM_EOL);
   make_stdcls (&class_uvec,		   "<uvec>",
 	       scm_class_class, class_bytevector,	   SCM_EOL);
-  make_stdcls (&class_array,		   "<array>",
-	       scm_class_class, scm_class_top,		   SCM_EOL);
-  make_stdcls (&class_bitvector,           "<bitvector>",
-	       scm_class_class, scm_class_top,		   SCM_EOL);
   make_stdcls (&scm_class_number,	   "<number>",
 	       scm_class_class, scm_class_top,		   SCM_EOL);
   make_stdcls (&scm_class_complex,	   "<complex>",
@@ -2777,21 +2755,13 @@ SCM_KEYWORD (k_getter, "getter");
 SCM
 scm_ensure_accessor (SCM name)
 {
-  SCM var, gf;
-
-  var = scm_module_variable (scm_current_module (), name);
-  if (SCM_VARIABLEP (var) && !SCM_UNBNDP (SCM_VARIABLE_REF (var)))
-    gf = SCM_VARIABLE_REF (var);
-  else
-    gf = SCM_BOOL_F;
-
+  SCM gf = scm_call_2 (SCM_TOP_LEVEL_LOOKUP_CLOSURE, name, SCM_BOOL_F);
   if (!SCM_IS_A_P (gf, scm_class_accessor))
     {
       gf = scm_make (scm_list_3 (scm_class_generic, k_name, name));
       gf = scm_make (scm_list_5 (scm_class_accessor,
 				 k_name, name, k_setter, gf));
     }
-
   return gf;
 }
 
