@@ -1,5 +1,4 @@
-/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2002, 2004, 2006,
- *   2007, 2008, 2009, 2011, 2013, 2014 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2002, 2004, 2006, 2007, 2008, 2009, 2011 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -29,18 +28,6 @@
 #include <stdio.h>
 #include <errno.h>
 
-#ifdef HAVE_PROCESS_H
-#include <process.h>    /* for mingw */
-#endif
-
-#include <unistd.h>
-
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
-
-#include <full-write.h>
-
 #include "libguile/_scm.h"
 
 #include "libguile/async.h"
@@ -52,6 +39,28 @@
 #include "libguile/validate.h"
 #include "libguile/scmsigs.h"
 
+#ifdef HAVE_PROCESS_H
+#include <process.h>    /* for mingw */
+#endif
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+
+#ifdef __MINGW32__
+#include <windows.h>
+#define alarm(sec) (0)
+/* This weird comma expression is because Sleep is void under Windows. */
+#define sleep(sec) (Sleep ((sec) * 1000), 0)
+#define usleep(usec) (Sleep ((usec) / 1000), 0)
+#endif
+
+#include <full-write.h>
+
 
 
 
@@ -109,10 +118,8 @@ static SIGRETTYPE (*orig_handlers[NSIG])(int);
 static SCM
 close_1 (SCM proc, SCM arg)
 {
-  /* Eval in the root module so that `lambda' has its usual meaning.  */
-  return scm_eval (scm_list_3 (scm_sym_lambda, SCM_EOL,
-                               scm_list_2 (proc, arg)),
-                   scm_the_root_module ());
+  return scm_primitive_eval_x (scm_list_3 (scm_sym_lambda, SCM_EOL,
+					   scm_list_2 (proc, arg)));
 }
 
 #if SCM_USE_PTHREAD_THREADS
@@ -492,7 +499,6 @@ SCM_DEFINE (scm_restore_signals, "restore-signals", 0, 0, 0,
 }
 #undef FUNC_NAME
 
-#if HAVE_DECL_ALARM
 SCM_DEFINE (scm_alarm, "alarm", 1, 0, 0,
            (SCM i),
 	    "Set a timer to raise a @code{SIGALRM} signal after the specified\n"
@@ -508,7 +514,6 @@ SCM_DEFINE (scm_alarm, "alarm", 1, 0, 0,
   return scm_from_uint (alarm (scm_to_uint (i)));
 }
 #undef FUNC_NAME
-#endif /* HAVE_ALARM */
 
 #ifdef HAVE_SETITIMER
 SCM_DEFINE (scm_setitimer, "setitimer", 5, 0, 0,
