@@ -1,5 +1,4 @@
-/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2003, 2004, 2005, 2006,
- *   2007, 2008, 2009, 2011, 2013, 2014 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2011 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -60,17 +59,38 @@
 #include "libguile/validate.h"
 #include "libguile/stime.h"
 
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#include <time.h>				  /* Gnulib-provided */
+#endif
 
-#include <sys/types.h>
+
+#ifdef HAVE_CLOCK_GETTIME
+# include <time.h>
+#endif
+
+#ifdef HAVE_SYS_TYPES_H
+# include <sys/types.h>
+#endif
+
+#ifdef HAVE_STRING_H
 #include <string.h>
-#include <sys/times.h>
+#endif
+
+#ifdef HAVE_SYS_TIMES_H
+# include <sys/times.h>
+#endif
 
 #ifdef HAVE_SYS_TIMEB_H
 # include <sys/timeb.h>
 #endif
 
+#ifndef tzname /* For SGI.  */
+extern char *tzname[]; /* RS6000 and others reject char **tzname.  */
+#endif
+#if defined (__MINGW32__)
+# define tzname _tzname
+#endif
+
 #if ! HAVE_DECL_STRPTIME
 extern char *strptime ();
 #endif
@@ -157,6 +177,7 @@ get_internal_real_time_gettimeofday (void)
 #endif
 
 
+#if defined HAVE_TIMES
 static long ticks_per_second;
 
 static long
@@ -167,6 +188,7 @@ get_internal_run_time_times (void)
   return (time_buffer.tms_utime + time_buffer.tms_stime)
     * TIME_UNITS_PER_SECOND / ticks_per_second;
 }
+#endif
 
 static timet fallback_real_time_base;
 static long
@@ -188,6 +210,7 @@ SCM_DEFINE (scm_get_internal_real_time, "get-internal-real-time", 0, 0, 0,
 #undef FUNC_NAME
 
 
+#ifdef HAVE_TIMES
 SCM_DEFINE (scm_times, "times", 0, 0, 0,
             (void),
 	    "Return an object with information about real and processor\n"
@@ -238,6 +261,7 @@ SCM_DEFINE (scm_times, "times", 0, 0, 0,
   return result;
 }
 #undef FUNC_NAME
+#endif /* HAVE_TIMES */
 
 long
 scm_c_get_internal_run_time (void)
@@ -327,7 +351,7 @@ filltime (struct tm *bd_time, int zoff, const char *zname)
   return result;
 }
 
-static const char tzvar[3] = "TZ";
+static char tzvar[3] = "TZ";
 
 /* if zone is set, create a temporary environment with only a TZ
    string.  other threads or interrupt handlers shouldn't be allowed
@@ -408,7 +432,7 @@ SCM_DEFINE (scm_localtime, "localtime", 1, 1, 0,
       const char *ptr;
 
       /* copy zone name before calling gmtime or restoring zone.  */
-#if defined (HAVE_STRUCT_TM_TM_ZONE)
+#if defined (HAVE_TM_ZONE)
       ptr = ltptr->tm_zone;
 #elif defined (HAVE_TZNAME)
       ptr = tzname[ (ltptr->tm_isdst == 1) ? 1 : 0 ];
@@ -517,7 +541,7 @@ bdtime2c (SCM sbd_time, struct tm *lt, int pos, const char *subr)
 #if HAVE_STRUCT_TM_TM_GMTOFF
   lt->tm_gmtoff = - scm_to_int (SCM_SIMPLE_VECTOR_REF (sbd_time, 9));
 #endif
-#ifdef HAVE_STRUCT_TM_TM_ZONE
+#ifdef HAVE_TM_ZONE
   if (scm_is_false (SCM_SIMPLE_VECTOR_REF (sbd_time, 10)))
     lt->tm_zone = NULL;
   else
@@ -527,14 +551,13 @@ bdtime2c (SCM sbd_time, struct tm *lt, int pos, const char *subr)
 
 SCM_DEFINE (scm_mktime, "mktime", 1, 1, 0,
             (SCM sbd_time, SCM zone),
-	    "@var{sbd_time} is an object representing broken down time and\n"
-	    "@code{zone} is an optional time zone specifier (otherwise the\n"
-	    "TZ environment variable or the system default is used).\n"
-	    "\n"
-	    "Returns a pair: the car is a corresponding integer time value\n"
-	    "like that returned by @code{current-time}; the cdr is a broken\n"
-	    "down time object, similar to as @var{sbd_time} but with\n"
-	    "normalized values.")
+	    "@var{bd-time} is an object representing broken down time and @code{zone}\n"
+	    "is an optional time zone specifier (otherwise the TZ environment variable\n"
+	    "or the system default is used).\n\n"
+	    "Returns a pair: the car is a corresponding\n"
+	    "integer time value like that returned\n"
+	    "by @code{current-time}; the cdr is a broken down time object, similar to\n"
+	    "as @var{bd-time} but with normalized values.")
 #define FUNC_NAME s_scm_mktime
 {
   timet itime;
@@ -568,7 +591,7 @@ SCM_DEFINE (scm_mktime, "mktime", 1, 1, 0,
       const char *ptr;
 
       /* copy zone name before calling gmtime or restoring the zone.  */
-#if defined (HAVE_STRUCT_TM_TM_ZONE)
+#if defined (HAVE_TM_ZONE)
       ptr = lt.tm_zone;
 #elif defined (HAVE_TZNAME)
       ptr = tzname[ (lt.tm_isdst == 1) ? 1 : 0 ];
@@ -678,7 +701,7 @@ SCM_DEFINE (scm_strftime, "strftime", 2, 0, 0,
 
   tbuf = scm_malloc (size);
   {
-#if !defined (HAVE_STRUCT_TM_TM_ZONE)
+#if !defined (HAVE_TM_ZONE)
     /* it seems the only way to tell non-GNU versions of strftime what
        zone to use (for the %Z format) is to set TZ in the
        environment.  interrupts and thread switching must be deferred
@@ -716,7 +739,7 @@ SCM_DEFINE (scm_strftime, "strftime", 2, 0, 0,
 	tbuf = scm_malloc (size);
       }
 
-#if !defined (HAVE_STRUCT_TM_TM_ZONE)
+#if !defined (HAVE_TM_ZONE)
     if (have_zone)
       {
 	restorezone (zone_spec, oldenv, FUNC_NAME);
@@ -740,7 +763,7 @@ SCM_DEFINE (scm_strptime, "strptime", 2, 0, 0,
             (SCM format, SCM string),
 	    "Performs the reverse action to @code{strftime}, parsing\n"
 	    "@var{string} according to the specification supplied in\n"
-	    "@var{format}.  The interpretation of month and day names is\n"
+	    "@var{template}.  The interpretation of month and day names is\n"
 	    "dependent on the current locale.  The value returned is a pair.\n"
 	    "The car has an object with time components\n"
 	    "in the form returned by @code{localtime} or @code{gmtime},\n"
@@ -852,6 +875,7 @@ scm_init_stime()
 
   /* Init ticks_per_second for scm_times, and use times(2)-based
      run-time timer if needed. */
+#ifdef HAVE_TIMES
 #ifdef _SC_CLK_TCK
   ticks_per_second = sysconf (_SC_CLK_TCK);
 #else
@@ -859,6 +883,7 @@ scm_init_stime()
 #endif
   if (!get_internal_run_time)
     get_internal_run_time = get_internal_run_time_times;
+#endif
 
   if (!get_internal_real_time)
     /* No POSIX timers, gettimeofday doesn't work... badness!  */
@@ -867,6 +892,10 @@ scm_init_stime()
       get_internal_real_time = get_internal_real_time_fallback;
     }
 
+  /* If we don't have a run-time timer, use real-time.  */
+  if (!get_internal_run_time)
+    get_internal_run_time = get_internal_real_time;
+
   scm_add_feature ("current-time");
 #include "libguile/stime.x"
 }
