@@ -1,18 +1,19 @@
-/* Copyright (C) 1999,2000,2001,2003,2004, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+/* Copyright (C) 1999,2000,2001,2003,2004, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 3 of
+ * the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301 USA
  */
 
 #if HAVE_CONFIG_H
@@ -21,8 +22,8 @@
 
 #include <libguile.h>
 
+#include <stdlib.h>
 #include <stdio.h>
-#include <assert.h>
 #include <string.h>
 
 #ifdef HAVE_INTTYPES_H
@@ -50,7 +51,7 @@ test_1 (const char *str, scm_t_intmax min, scm_t_intmax max,
       fprintf (stderr, "fail: scm_is_signed_integer (%s, "
 	       "%" PRIiMAX ", %" PRIiMAX ") == %d\n",
 	       str, min, max, result);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 }
 
@@ -136,7 +137,7 @@ test_2 (const char *str, scm_t_uintmax min, scm_t_uintmax max,
       fprintf (stderr, "fail: scm_is_unsigned_integer (%s, "
 	       "%" PRIuMAX ", %" PRIuMAX ") == %d\n",
 	       str, min, max, result);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 }
 
@@ -257,7 +258,7 @@ test_3 (const char *str, scm_t_intmax min, scm_t_intmax max,
 		   "fail: scm_to_signed_int (%s, "
 		   "%" PRIiMAX ", %" PRIiMAX ") -> out of range\n",
 		   str, min, max);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
   else if (type_error)
@@ -270,7 +271,7 @@ test_3 (const char *str, scm_t_intmax min, scm_t_intmax max,
 		   "fail: scm_to_signed_int (%s, "
 		   "%" PRIiMAX", %" PRIiMAX ") -> wrong type\n",
 		   str, min, max);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
   else
@@ -284,7 +285,7 @@ test_3 (const char *str, scm_t_intmax min, scm_t_intmax max,
 		   "fail: scm_to_signed_int (%s, "
 		   "%" PRIiMAX ", %" PRIiMAX ") = %" PRIiMAX "\n",
 		   str, min, max, result);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
 }
@@ -392,7 +393,7 @@ test_4 (const char *str, scm_t_uintmax min, scm_t_uintmax max,
 		   "fail: scm_to_unsigned_int (%s, "
 		   "%" PRIuMAX ", %" PRIuMAX ") -> out of range\n",
 		   str, min, max);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
   else if (type_error)
@@ -405,7 +406,7 @@ test_4 (const char *str, scm_t_uintmax min, scm_t_uintmax max,
 		   "fail: scm_to_unsigned_int (%s, "
 		   "%" PRIuMAX ", %" PRIuMAX ") -> wrong type\n",
 		   str, min, max);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
   else
@@ -419,7 +420,7 @@ test_4 (const char *str, scm_t_uintmax min, scm_t_uintmax max,
 		   "fail: scm_to_unsigned_int (%s, "
 		   "%" PRIuMAX ", %" PRIuMAX ") == %" PRIuMAX "\n",
 		   str, min, max, result);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
 }
@@ -475,7 +476,7 @@ test_5 (scm_t_intmax val, const char *result)
     {
       fprintf (stderr, "fail: scm_from_signed_integer (%" PRIiMAX ") == %s\n",
 	       val, result);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 }
 
@@ -508,7 +509,7 @@ test_6 (scm_t_uintmax val, const char *result)
       fprintf (stderr, "fail: scm_from_unsigned_integer (%"
 	       PRIuMAX ") == %s\n",
 	       val, result);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 }
 
@@ -536,7 +537,7 @@ test_7s (SCM n, scm_t_intmax c_n, const char *result, const char *func)
   if (scm_is_false (scm_equal_p (n, r)))
     {
       fprintf (stderr, "fail: %s (%" PRIiMAX ") == %s\n", func, c_n, result);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 }
 
@@ -550,7 +551,7 @@ test_7u (SCM n, scm_t_uintmax c_n, const char *result, const char *func)
   if (scm_is_false (scm_equal_p (n, r)))
     {
       fprintf (stderr, "fail: %s (%" PRIuMAX ") == %s\n", func, c_n, result);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 }
 
@@ -586,7 +587,7 @@ test_8s (const char *str, scm_t_intmax (*func) (SCM), const char *func_name,
 	{
 	  fprintf (stderr,
 		   "fail: %s (%s) -> out of range\n", func_name, str);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
   else if (type_error)
@@ -597,7 +598,7 @@ test_8s (const char *str, scm_t_intmax (*func) (SCM), const char *func_name,
 	{
 	  fprintf (stderr,
 		   "fail: %s (%s) -> wrong type\n", func_name, str);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
   else
@@ -609,7 +610,7 @@ test_8s (const char *str, scm_t_intmax (*func) (SCM), const char *func_name,
 	{
 	  fprintf (stderr,
 		   "fail: %s (%s) = %" PRIiMAX "\n", func_name, str, result);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
 }
@@ -644,7 +645,7 @@ test_8u (const char *str, scm_t_uintmax (*func) (SCM), const char *func_name,
 	{
 	  fprintf (stderr,
 		   "fail: %s (%s) -> out of range\n", func_name, str);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
   else if (type_error)
@@ -655,7 +656,7 @@ test_8u (const char *str, scm_t_uintmax (*func) (SCM), const char *func_name,
 	{
 	  fprintf (stderr,
 		   "fail: %s (%s) -> wrong type\n", func_name, str);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
   else
@@ -667,7 +668,7 @@ test_8u (const char *str, scm_t_uintmax (*func) (SCM), const char *func_name,
 	{
 	  fprintf (stderr,
 		   "fail: %s (%s) = %" PRIiMAX "\n", func_name, str, result);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
 }
@@ -702,10 +703,8 @@ DEFSTST (scm_to_int16)
 DEFUTST (scm_to_uint16)
 DEFSTST (scm_to_int32)
 DEFUTST (scm_to_uint32)
-#ifdef SCM_HAVE_T_INT64
 DEFSTST (scm_to_int64)
 DEFUTST (scm_to_uint64)
-#endif
 
 #define TEST_8S(v,f,r,re,te) test_8s (v, tst_##f, #f, r, re, te)
 #define TEST_8U(v,f,r,re,te) test_8u (v, tst_##f, #f, r, re, te)
@@ -745,11 +744,9 @@ test_int_sizes ()
   TEST_7S (scm_from_int32,  SCM_T_INT32_MAX+1LL, "-2147483648");
   TEST_7U (scm_from_uint32, SCM_T_UINT32_MAX,     "4294967295");
 
-#if SCM_HAVE_T_INT64
   TEST_7S (scm_from_int64,  SCM_T_INT64_MIN,  "-9223372036854775808");
   TEST_7S (scm_from_int64,  SCM_T_INT64_MAX,   "9223372036854775807");
   TEST_7U (scm_from_uint64, SCM_T_UINT64_MAX, "18446744073709551615");
-#endif
 
   TEST_8S ("91",   scm_to_schar,   91, 0, 0);
   TEST_8U ("91",   scm_to_uchar,   91, 0, 0);
@@ -794,7 +791,6 @@ test_int_sizes ()
   TEST_8U ("-1",          scm_to_uint32,                0, 1, 0);
   TEST_8U ("#f",          scm_to_uint32,                0, 0, 1);
 
-#if SCM_HAVE_T_INT64
   TEST_8S ("-9223372036854775808", scm_to_int64,   SCM_T_INT64_MIN, 0, 0);
   TEST_8S ("9223372036854775807",  scm_to_int64,   SCM_T_INT64_MAX, 0, 0);
   TEST_8S ("9223372036854775808",  scm_to_int64,                 0, 1, 0);
@@ -803,7 +799,6 @@ test_int_sizes ()
   TEST_8U ("18446744073709551616", scm_to_uint64,                0, 1, 0);
   TEST_8U ("-1",                   scm_to_uint64,                0, 1, 0);
   TEST_8U ("#f",                   scm_to_uint64,                0, 0, 1);
-#endif
 
 }
 
@@ -814,7 +809,7 @@ test_9 (double val, const char *result)
   if (scm_is_false (scm_eqv_p (res, scm_from_double (val))))
     {
       fprintf (stderr, "fail: scm_from_double (%g) == %s\n", val, result);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 }
 
@@ -833,7 +828,7 @@ ieee_init (void)
      before trying to use it.  (But in practice we believe this is not a
      problem on any system guile is likely to target.)  */
   guile_Inf = INFINITY;
-#elif HAVE_DINFINITY
+#elif defined HAVE_DINFINITY
   /* OSF */
   extern unsigned int DINFINITY[2];
   guile_Inf = (*((double *) (DINFINITY)));
@@ -852,7 +847,7 @@ ieee_init (void)
 #ifdef NAN
   /* C99 NAN, when available */
   guile_NaN = NAN;
-#elif HAVE_DQNAN
+#elif defined HAVE_DQNAN
   {
     /* OSF */
     extern unsigned int DQNAN[2];
@@ -901,7 +896,7 @@ test_10 (const char *val, double result, int type_error)
 	{
 	  fprintf (stderr,
 		   "fail: scm_double (%s) -> wrong type\n", val);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
   else
@@ -913,7 +908,7 @@ test_10 (const char *val, double result, int type_error)
 	{
 	  fprintf (stderr,
 		   "fail: scm_to_double (%s) = %g\n", val, result);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
 }
@@ -958,7 +953,7 @@ test_11 (const char *str, const char *result, int misc_error, int type_error)
 	{
 	  fprintf (stderr,
 		   "fail: scm_to_locale_string (%s) -> misc error\n", str);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
   else if (type_error)
@@ -969,7 +964,7 @@ test_11 (const char *str, const char *result, int misc_error, int type_error)
 	{
 	  fprintf (stderr,
 		   "fail: scm_to_locale_string (%s) -> wrong type\n", str);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
   else
@@ -981,7 +976,7 @@ test_11 (const char *str, const char *result, int misc_error, int type_error)
 	{
 	  fprintf (stderr,
 		   "fail: scm_to_locale_string (%s) = %s\n", str, result);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
 
@@ -1000,7 +995,7 @@ test_locale_strings ()
   if (!scm_is_string (scm_c_eval_string ("\"foo\"")))
     {
       fprintf (stderr, "fail: scm_is_string (\"foo\") = true\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   str = scm_from_locale_string (lstr);
@@ -1008,14 +1003,14 @@ test_locale_strings ()
   if (!scm_is_string (str))
     {
       fprintf (stderr, "fail: scm_is_string (str) = true\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   lstr2 = scm_to_locale_string (str);
   if (strcmp (lstr, lstr2))
     {
       fprintf (stderr, "fail: lstr = lstr2\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   free (lstr2);
 
@@ -1024,17 +1019,17 @@ test_locale_strings ()
   if (len != strlen (lstr))
     {
       fprintf (stderr, "fail: scm_to_locale_stringbuf (...) = strlen(lstr)\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   if (buf[15] != 'x')
     {
       fprintf (stderr, "fail: scm_to_locale_stringbuf (...) no overrun\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   if (strncmp (lstr, buf, 15))
     {
       fprintf (stderr, "fail: scm_to_locale_stringbuf (...) = lstr\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   str2 = scm_from_locale_stringn (lstr, 10);
@@ -1042,14 +1037,14 @@ test_locale_strings ()
   if (!scm_is_string (str2))
     {
       fprintf (stderr, "fail: scm_is_string (str2) = true\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   lstr2 = scm_to_locale_string (str2);
   if (strncmp (lstr, lstr2, 10))
     {
       fprintf (stderr, "fail: lstr = lstr2\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   free (lstr2);
 
@@ -1058,24 +1053,24 @@ test_locale_strings ()
   if (len != 10)
     {
       fprintf (stderr, "fail: scm_to_locale_stringbuf (...) = 10\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   if (buf[10] != 'x')
     {
       fprintf (stderr, "fail: scm_to_locale_stringbuf (...) no overrun\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   if (strncmp (lstr, buf, 10))
     {
       fprintf (stderr, "fail: scm_to_locale_stringbuf (...) = lstr\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   lstr2 = scm_to_locale_stringn (str2, &len);
   if (len != 10)
     {
       fprintf (stderr, "fail: scm_to_locale_stringn, len = 10\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   test_11 ("#f", NULL, 0, 1);
@@ -1083,6 +1078,37 @@ test_locale_strings ()
   test_11 ("(string #\\f #\\nul)", NULL, 1, 0);
 }
 
+static void
+test_is_exact ()
+{
+  if (1 != scm_is_exact (scm_c_eval_string ("3")))
+    {
+      fprintf (stderr, "fail: scm_is_exact (\"3\") = 1\n");
+      exit (EXIT_FAILURE);
+    }
+  if (0 != scm_is_exact (scm_c_eval_string ("3.0")))
+    {
+      fprintf (stderr, "fail: scm_is_exact (\"3.0\") = 0\n");
+      exit (EXIT_FAILURE);
+    }
+}
+
+static void
+test_is_inexact ()
+{
+  if (1 !=scm_is_inexact (scm_c_eval_string ("3.0")))
+    {
+      fprintf (stderr, "fail: scm_is_inexact (\"3.0\") = 1\n");
+      exit (EXIT_FAILURE);
+    }
+  if (0 != scm_is_inexact (scm_c_eval_string ("3")))
+    {
+      fprintf (stderr, "fail: scm_is_inexact (\"3\") = 0\n");
+      exit (EXIT_FAILURE);
+    }
+}
+
+
 static void
 tests (void *data, int argc, char **argv)
 {
@@ -1096,6 +1122,8 @@ tests (void *data, int argc, char **argv)
   test_from_double ();
   test_to_double ();
   test_locale_strings ();
+  test_is_exact ();
+  test_is_inexact ();
 }
 
 int
